-- GLOBAL --
ATT = select(2, ...) 
-- MODULE --
local _G = getfenv(0)
setmetatable(ATT, {__index = _G})
setfenv(1, ATT)
-- LOCAL --
title		= "Alt-Tab Toggle"
folder		= "Alt-Tab_Toggle"
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version

coreFrame = CreateFrame("Frame", nil, UIParent)-- Used in Alt-Tab Detection.
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)
local P --db.profile
local G
local LSM = LibStub("LibSharedMedia-3.0")


lastOnUpdate = 0 --don't touch
ATTActive = false --don't touch

toggleThrottle = 5 --don't toggle if we've toggled in the last x seconds.

directory	= "Interface\\Addons\\"..folder.."\\"
timers = {}





--Globals
local CLEARED_AFK_							= _G.CLEARED_AFK
local COMBATLOG_OBJECT_REACTION_HOSTILE_	= _G.COMBATLOG_OBJECT_REACTION_HOSTILE
local COMBATLOG_OBJECT_TYPE_PLAYER_			= _G.COMBATLOG_OBJECT_TYPE_PLAYER
local COMBAT_								= _G.COMBAT
local ConsoleExec_							= _G.ConsoleExec
local ENABLE_								= _G.ENABLE
local ENTER_DUNGEON_						= _G.ENTER_DUNGEON
local EVENTS_LABEL_							= _G.EVENTS_LABEL
local FACTION_INACTIVE_						= _G.FACTION_INACTIVE
local GAMEOPTIONS_MENU_						= _G.GAMEOPTIONS_MENU
local GetBattlefieldStatus_					= _G.GetBattlefieldStatus
local GetCVar_								= _G.GetCVar
local GetRealZoneText_						= _G.GetRealZoneText
local GetTime_								= _G.GetTime
local INVITE_								= _G.INVITE
local InCombatLockdown_						= _G.InCombatLockdown
local InterfaceOptionsFrame_OpenToCategory_	= _G.InterfaceOptionsFrame_OpenToCategory
local IsInInstance_							= _G.IsInInstance
local MARKED_AFK_							= _G.MARKED_AFK
local MAX_BATTLEFIELD_QUEUES_				= _G.MAX_BATTLEFIELD_QUEUES
local PlaySoundFile_						= _G.PlaySoundFile
local READY_CHECK_							= _G.READY_CHECK
local SUMMON_								= _G.SUMMON
local SetCVar_								= _G.SetCVar
local UnitDebuff_							= _G.UnitDebuff
local UnitIsDND_							= _G.UnitIsDND
local WHISPER_								= _G.WHISPER
local bit_band								= _G.bit.band
local pairs_								= _G.pairs
local print_								= _G.print
local select_								= _G.select
local table_getn							= _G.table.getn
local tostring_								= _G.tostring
local GetFramerate_							= _G.GetFramerate
local GetNumPartyMembers_					= GetNumPartyMembers
local GetNumRaidMembers_					= GetNumRaidMembers
local tonumber_								= tonumber
local StaticPopup_Show_						= StaticPopup_Show
local CHAT_CONFIGURATION_					= _G.CHAT_CONFIGURATION
local UnitOnTaxi_							= _G.UnitOnTaxi


-- Default Settings
defaultSettings = {profile = {}}
defaultSettings.profile.ATTBGGatesOpening		= true
defaultSettings.profile.ATTBGPop				= true
defaultSettings.profile.ATTCombat				= true
defaultSettings.profile.ATTDeserter				= true
defaultSettings.profile.ATTEnemyDetected		= false
defaultSettings.profile.ATTFlight				= true
defaultSettings.profile.ATTIdle					= true
defaultSettings.profile.ATTInvite				= true
defaultSettings.profile.ATTReadyCheck			= true
defaultSettings.profile.ATTSummon				= true
defaultSettings.profile.ATTWhisper				= true
defaultSettings.profile.attAFKActivate			= false
defaultSettings.profile.attAlertSound			= true
defaultSettings.profile.attAltTabDetection		= true
defaultSettings.profile.soundOnly				= false
defaultSettings.profile.ATTEnterDungeonQueue	= true
defaultSettings.profile.ATTBootProposal			= true
defaultSettings.profile.ATTRoleCheck			= true
defaultSettings.profile.ignoreWhenDND			= true
defaultSettings.profile.alwaysActivated			= false
defaultSettings.profile.enableSounds			= true
defaultSettings.profile.flashScreen				= false

defaultSettings.profile.fullParty			= true
defaultSettings.profile.fullRaid			= true
defaultSettings.profile.wintergraspEntry			= true
defaultSettings.profile.dungeonCooldown			= true
defaultSettings.profile.mainSound = "ATT alert"


defaultSettings.profile.sounds = {}
defaultSettings.profile.sounds.ATTIdle = "ATT alert"
defaultSettings.profile.sounds.ATTDeserter = "ATT alert"
defaultSettings.profile.sounds.ATTWhisper = "ATT alert"
defaultSettings.profile.sounds.ATTCombat = "ATT alert"
defaultSettings.profile.sounds.ATTFlight = "ATT alert"
defaultSettings.profile.sounds.ATTReadyCheck = "ATT alert"
defaultSettings.profile.sounds.ATTSummon = "ATT alert"
defaultSettings.profile.sounds.ATTInvite = "ATT alert"
defaultSettings.profile.sounds.ATTBGPop = "ATT alert"
defaultSettings.profile.sounds.ATTEnemyDetected = "ATT alert"
defaultSettings.profile.sounds.ATTBGGatesOpening = "ATT alert"
defaultSettings.profile.sounds.ATTEnterDungeonQueue = "ATT alert"
defaultSettings.profile.sounds.ATTBootProposal = "ATT alert"
defaultSettings.profile.sounds.ATTRoleCheck = "ATT alert"
defaultSettings.profile.sounds.fullParty = "ATT alert"
defaultSettings.profile.sounds.fullRaid = "ATT alert"
defaultSettings.profile.sounds.wintergraspEntry = "ATT alert"
defaultSettings.profile.sounds.dungeonCooldown = "ATT alert"

defaultSettings.global = {}
defaultSettings.global.soundFiles = {
	["ATT alert"] = directory.."Sounds\\ATTalert.mp3", --	/run PlaySoundFile_(ATT.directory.."Sounds\\ATTalert.mp3")
	["BellTollAlliance"] = "Sound\\Doodad\\BellTollAlliance.wav",--	/run PlaySoundFile_("Sound\\Doodad\\BellTollAlliance.wav")
	["BellTollHorde"] = "Sound\\Doodad\\BellTollHorde.wav",
	["BellTollNightElf"] = "Sound\\Doodad\\BellTollNightElf.wav",
	["BellTollTribal"] = "Sound\\Doodad\\BellTollTribal.wav",
	["BoatDockedWarning"] = "Sound\\Doodad\\BoatDockedWarning.wav",
	["G_GongTroll01"] = "Sound\\Doodad\\G_GongTroll01.wav",
	["ShaysBell"] = "Sound\\Spells\\ShaysBell.wav",
	["PVPEnterQueue"] = "Sound\\Spells\\PVPEnterQueue.wav",
	["bind2_Impact_Base"] = "Sound\\Spells\\bind2_Impact_Base.wav",
	["KharazahnBellToll"] = "Sound\\Doodad\\KharazahnBellToll.wav",
	["AuctionWindowOpen"] = "Sound\\Interface\\AuctionWindowOpen.wav",
	["AuctionWindowClose"] = "Sound\\Interface\\AuctionWindowClose.wav",
	["AlarmClockWarning1"] = "Sound\\Interface\\AlarmClockWarning1.wav",
	["AlarmClockWarning2"] = "Sound\\Interface\\AlarmClockWarning2.wav",
	["AlarmClockWarning3"] = "Sound\\Interface\\AlarmClockWarning3.wav",
	["MapPing"] = "Sound\\Interface\\MapPing.wav",
	["SimonGame_Visual_GameTick"] = "Sound\\Spells\\SimonGame_Visual_GameTick.wav",
	["SimonGame_Visual_LevelStart"] = "Sound\\Spells\\SimonGame_Visual_LevelStart.wav",
	["SimonGame_Visual_GameStart"] = "Sound\\Spells\\SimonGame_Visual_GameStart.wav",
	["YarrrrImpact"] = "Sound\\Spells\\YarrrrImpact.wav",
--~ 	[""] = "",

}
--

-- Echo Strings
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.


--Internal settings
local atDetectionWait	= 5-- if our OnUpdate function hasn't updated in 5 seconds, assume we've alt tabbed.
ignoreEnemyZones = {
	[L["Shattrath City"]]	= true,
	[L["Dalaran"]] 			= true,
	[""]					= true,--Blank when we login, the GetZone API doesn't return a zone name.
}

local regEvents = {
	"CHAT_MSG_BG_SYSTEM_NEUTRAL",
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_WHISPER",
	"COMBAT_LOG_EVENT_UNFILTERED",
	"CONFIRM_SUMMON",
	"PARTY_INVITE_REQUEST",
	"PLAYER_CONTROL_GAINED",
--~ 	"PLAYER_CONTROL_LOST",
	"PLAYER_ENTERING_WORLD",
	"PLAYER_REGEN_DISABLED",
	"READY_CHECK",
	"UNIT_AURA",
	"UPDATE_BATTLEFIELD_STATUS",
	"LFG_PROPOSAL_SHOW",
	"LFG_ROLE_CHECK_SHOW",
	"LFG_BOOT_PROPOSAL_UPDATE",
	"PARTY_MEMBERS_CHANGED",
	"RAID_ROSTER_UPDATE",
	"BATTLEFIELD_MGR_ENTRY_INVITE",
}

local lastPartyCount = 0
local lastRaidCount = 0


-- Ace3 Functions
function core:OnInitialize()--	/script ATT.core.OnInitialize()
	db = LibStub("AceDB-3.0"):New("ATT_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	core:RegisterChatCommand("att", "MySlashProcessorFunc")

	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, "ATT "..version)
end

maxFPSBk = 10--default is 10FPS while alt-tabbed in window mode.
function core:OnEnable()
	P = db.profile
	G = db.global

	if GetCVar_("gxWindow") == "1" then
		maxFPSBk = tonumber_(GetCVar_("maxFPSBk"))
		
		coreFrame.lU = 0
		coreFrame:SetScript("OnUpdate", function(this, elapsed)
			this.lU = this.lU + elapsed
			if this.lU > 1 then
				this.lU = 0
				
				WindowModeCheckFPS()
			end
		end)
		
	else
		coreFrame:SetScript("OnUpdate", function(this, elapsed)
			lastOnUpdate = GetTime_()
		end)
	end
	coreFrame:Show()
	
	for i, event in pairs_(regEvents) do 
		self:RegisterEvent(event)
	end
	
	if GetCVar_("Sound_EnableSoundWhenGameIsInBG") == "0" then
		print_(L["'Sounds in Background' is disabled, Enable it if you want sounds when alt tabbed."])
	end
	
	--make all options clickable.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = false
		end
	end
	
	if not db.global.firstLogin then
		db.global.firstLogin = true
		StaticPopup_Show_("ATT_ConfirmToggling")
	end

--~ 	for sound, path in pairs(soundFiles) do 
--~ 		LSM:Register("sound", sound, path)
--~ 	end
	for sound, path in pairs(G.soundFiles) do 
		LSM:Register("sound", sound, path)
	end
	
	if P.alwaysActivated == true then
		ATTToggleMode(true, nil, L["Always Activated"])
	end
--~ 	for i = 1, table.getn(soundFiles) do 
--~ 		LSM:Register("sound", soundFiles[i].name, soundFiles[i].path)
--~ 	end
end


local prevFPS = {}
local lowFPS = false



function WindowModeCheckFPS()
	if InCombatLockdown_() then
		return
	end
--~ 	Debug("WindowModeCheckFPS", GetFramerate_())
	table.insert(prevFPS,1,GetFramerate_())
	for i=table.getn(prevFPS),5,-1  do 
		table.remove(prevFPS, i)
	end
end

function IsAltTabbed()
	if InCombatLockdown_() then
		--we're in combat, probably not alt tabbed. note this doesn't return true when we enter combat.
		return false
	end
--~ 	Debug("IsAltTabbed", GetCVar_("gxWindow"), type(GetCVar_("gxWindow")))
	if GetCVar_("gxWindow") == "0" then -- fullscreen mode.
		if GetTime_() - lastOnUpdate > atDetectionWait then
			return true
		end	
	else --window mode.
		if table.getn(prevFPS) > 0 then
			local total = 0
			for i=1, table.getn(prevFPS) do 
				total = total + prevFPS[i]
			end
			local avgFPS = total / table.getn(prevFPS)
--~ 			print(GetFramerate(), "total:"..total, "avgFPS:"..avgFPS, "maxFPSBk:"..maxFPSBk)
			
			if avgFPS <= (maxFPSBk+1) then
				return true
			end
		end
	end
	return false
end

function sayHi()--	/run ATT.sayHi()
	local myMsg= "hello"
	print(myMsg)
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /att function brings up the UI options.	--
----------------------------------------------
	if core:IsEnabled() then
		if not input or input == "" then
			ATTToggleMode(nil, nil, "Slash command")
			return
		end
	else
		print_(L["ATT is disabled, enable it on the options screen first."])
	end
	InterfaceOptionsFrame_OpenToCategory_(coreOpts)
end



function core:OnDisable()
	coreFrame:SetScript("OnUpdate", nil)
	
	--make all the options unclickable except for the enable button.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = true
		end
	end
	ATTToggleMode(false, false, L["Addon Disabled"])
end



----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end




function core:CHAT_MSG_BG_SYSTEM_NEUTRAL(event, ...)
	local message  = ...-- ***
	message = message:lower()
	if	message:find(L["the battle for arathi basin has begun!"]) or --AB
		message:find(L["the battle for alterac valley has begun"]) or --AV
		message:find(L["let the battle for warsong gulch begin!"]) or --WSG
		message:find(L["the battle has begun"]) then --EotS

		if P.ATTBGGatesOpening == true then
			FocusWoW(L["Gates Opening"], LSM:Fetch('sound', P.sounds.ATTBGGatesOpening))
		end

	end
end

function core:CONFIRM_SUMMON(event, ...)
	if P.ATTSummon == true then
		FocusWoW(SUMMON_, LSM:Fetch('sound', P.sounds.ATTSummon))
	end
end

function core:READY_CHECK(event, ...)
	if P.ATTReadyCheck == true then
		FocusWoW(L["Ready Check"], LSM:Fetch('sound', P.sounds.ATTReadyCheck))
	end
end


function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	if ignoreEnemyZones[GetRealZoneText_()] or InBattleground() then
		return
	end

	if P.ATTEnemyDetected == true then
		local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = ...
		
		if FlagIsPlayer(srcFlags) and FlagIsEnemy(srcFlags) then
			FocusWoW(L["Enemy detected (%s)"]:format(srcName), LSM:Fetch('sound', P.sounds.ATTEnemyDetected))
			
		elseif FlagIsPlayer(dstFlags) and FlagIsEnemy(dstFlags) then
			FocusWoW(L["Enemy detected (%s)"]:format(dstName), LSM:Fetch('sound', P.sounds.ATTEnemyDetected))
		end
	end
end

function FlagIsEnemy(flags)
	if bit_band(flags, COMBATLOG_OBJECT_REACTION_HOSTILE_) == COMBATLOG_OBJECT_REACTION_HOSTILE_ then
		return true
	end
	return nil
end

function FlagIsPlayer(flags)
	if bit_band(flags, COMBATLOG_OBJECT_TYPE_PLAYER_) == COMBATLOG_OBJECT_TYPE_PLAYER_ then
		return true
	end
	return nil
end

function core:PLAYER_ENTERING_WORLD(event, ...)
	lastFocusTime = GetTime_()--if we exit a BG, PLAYER_CONTROL_GAINED (flight land) will fire, we don't want to toggle WoW.
end

function core:PLAYER_REGEN_DISABLED(event, ...)
	if P.ATTCombat == true then
		FocusWoW(L["Enter combat"], LSM:Fetch('sound', P.sounds.ATTCombat))
	end
end

function core:CHAT_MSG_WHISPER(event, ...)
	if P.ATTWhisper == true then
		FocusWoW(L["Receive whisper"], LSM:Fetch('sound', P.sounds.ATTWhisper))
	end
end

function core:PARTY_INVITE_REQUEST(event, ...)
	if P.ATTInvite == true then
		FocusWoW(INVITE_, LSM:Fetch('sound', P.sounds.ATTInvite))
	end
end

function core:PLAYER_CONTROL_GAINED(event, ...)
--~ 	Debug(event, UnitOnTaxi("player"))
	if P.ATTFlight == true and UnitOnTaxi_("player") then
		FocusWoW(L["Flight land"], LSM:Fetch('sound', P.sounds.ATTFlight))
	end
end

function core:PLAYER_CONTROL_LOST(event, ...)
--~ 	Debug(event, UnitOnTaxi("player"))
end


function core:LFG_ROLE_CHECK_SHOW(event, ...)
	if P.ATTRoleCheck == true then
		FocusWoW(L["Role Check"], LSM:Fetch('sound', P.sounds.ATTRoleCheck))
	end
end

function core:LFG_PROPOSAL_SHOW(event, ...)
	if P.ATTEnterDungeonQueue == true then
		FocusWoW(L["Enter random dungeon"], LSM:Fetch('sound', P.sounds.ATTEnterDungeonQueue))
	end
end


function core:LFG_BOOT_PROPOSAL_UPDATE(event, ...)
	if P.ATTBootProposal == true then
		FocusWoW(L["Boot Proposal"], LSM:Fetch('sound', P.sounds.ATTBootProposal))
	end
end


function core:UPDATE_BATTLEFIELD_STATUS(event, ...)
	if P.ATTBGPop == true then
		local status, mapName, instanceID
		for i=1, MAX_BATTLEFIELD_QUEUES_ do
			status, mapName, instanceID = GetBattlefieldStatus_(i)
			if status == "confirm" then
				FocusWoW(L["BG confirm"], LSM:Fetch('sound', P.sounds.ATTBGPop))
				break
			end
		end
	end
end


function core:CHAT_MSG_SYSTEM(event, ...)
	if P.attAFKActivate == true then
		local message, format  = ...
		if message:find(MARKED_AFK_) then --You are now AFK: Away from Keyboard
			ATTToggleMode(true, nil, L["User went AFK"])
		elseif message:find(CLEARED_AFK_) then
			ATTToggleMode(false, nil, L["User no longer AFK"])
		end
	end
end


isDeserter = false
dungeonCooldown = false
dungeonDeserter = false
function core:UNIT_AURA(event, ...)
	local unitID  = ...
	if unitID == "player" then
		if P.ATTDeserter == true then
			local buffOn = IsDebuffOn(L["Deserter"], unitID)
			if isDeserter == false then
				if buffOn then
					isDeserter = true
					return
				end
			else
				if not buffOn then
					FocusWoW(L["Deserter Expire"], LSM:Fetch('sound', P.sounds.ATTDeserter))
					isDeserter = false
					return
				end
			end
		end
		
		if P.ATTIdle == true then
			if IsDebuffOn(L["Idle"], unitID) then
				FocusWoW(L["Idle debuff gained"], LSM:Fetch('sound', P.sounds.ATTIdle))
				return
			elseif IsDebuffOn(FACTION_INACTIVE_, unitID) then
				FocusWoW(L["Inactive debuff gained"], LSM:Fetch('sound', P.sounds.ATTIdle))
				return
			end
		end
		
		if P.dungeonCooldown == true then
			local buffOn = IsDebuffOn(L["Dungeon Cooldown"], unitID)
			if dungeonCooldown == false then
				if buffOn then
					dungeonCooldown = true
--~ 					Debug("We have Dungeon Cooldown")
					return
				end
			else
				if not buffOn then
					FocusWoW(L["Dungeon Cooldown"], LSM:Fetch('sound', P.sounds.dungeonCooldown))
					dungeonCooldown = false
--~ 					Debug("We lost Dungeon Cooldown")
					return
				end
			end
			
			
			local buffOn = IsDebuffOn(L["Dungeon Deserter"], unitID)
			if dungeonDeserter == false then
				if buffOn then
					dungeonDeserter = true
--~ 					Debug("We have Dungeon Deserter")
					return
				end
			else
				if not buffOn then
					FocusWoW(L["Dungeon Cooldown"], LSM:Fetch('sound', P.sounds.dungeonCooldown))
					dungeonDeserter = false
--~ 					Debug("We lost Dungeon Deserter")
					return
				end
			end
		end
	end
end

function core:PARTY_MEMBERS_CHANGED(event,...)
	if P.fullParty == true then
		if GetNumPartyMembers_() == 4 and GetNumRaidMembers_() == 0 then
--~ 			Debug(event, "full party", GetNumRaidMembers_())
			if GetNumPartyMembers_() > lastPartyCount then
				FocusWoW(L["Full Party"], LSM:Fetch('sound', P.sounds.fullParty))
			end
		end
		lastPartyCount = GetNumPartyMembers_()
	end
end

function core:RAID_ROSTER_UPDATE()
	if P.fullRaid == true then
		if GetNumRaidMembers_() == 10 or GetNumRaidMembers_() == 25 or GetNumRaidMembers_() == 40 then
			if GetNumRaidMembers_() > lastRaidCount then
				FocusWoW(L["Full Raid"], LSM:Fetch('sound', P.sounds.fullRaid))
			end
		end
		lastRaidCount = GetNumRaidMembers_()
	end
end
function core:BATTLEFIELD_MGR_ENTRY_INVITE(event, ...)
--~ 	local battleID = ... 
	if P.wintergraspEntry == true then
		FocusWoW(L["Wintergrasp Entry"], LSM:Fetch('sound', P.sounds.wintergraspEntry))
	end
--~ 	Debug(event, ...)
end

------------------------------------------
function IsDebuffOn(spellName, unitID)	--
-- Check if a spell exists on a unitID.	--
------------------------------------------
	local name
	for i = 1, 40 do
		name  =  UnitDebuff_(unitID, i)
		if not name then break end
		if name == spellName then
			return true
		end
	end
	return false
end

------------------------------
function Debug(from, ...)	--	/script DA.Debug("hey","bye")
-- simple print function.	--
------------------------------
	if version ~= "" then--Dev version shows no version.
		return
	end
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i])
	end
	print_(strDebugFrom:format(from).." "..msg)
end


--------------------------------------------------
function FocusWoW(reason, sound)				--
-- Check if ATT is active, if so toggle WoW.	--
--------------------------------------------------
	if P.ignoreWhenDND == true and UnitIsDND_("player") then
		return
	end

	if ATTActive == true or (P.attAltTabDetection == true and IsAltTabbed()) then --
		HardFocusWoW(reason, true, sound)
	end
end



lastFocusTime = 0
--------------------------------------------------------------
function HardFocusWoW(reason, force, sound)					--	/script ATT.HardFocusWoW()
-- Toggle WoW if we haven't toggled in the last 5 seconds.	--
--------------------------------------------------------------
	if GetTime_() - lastFocusTime < toggleThrottle then --don't toggle if we've already toggled in the past 5 seconds.
		return
	end
	if ATTActive == true or force == true then
--~ 		local WindowMode = GetCVar_("gxWindow")	--1 = windowmode, 0 = fullscreen.
--~ 		if (WindowMode == 1) then
--~ 			SetCVar_("gxWindow", "1") --window mode
--~ 		else
--~ 			SetCVar_("gxWindow", "0") --window full
--~ 		end
			
		if P.attAlertSound == true then -- and P.ATTDelayFocus == false
			if P.enableSounds == true then
				TurnSoundsOn()
				core:ScheduleTimer("TurnSoundsOff", 2)--turn sounds off in 2 seconds.
			end
			PlaySoundFile_(sound)
			
		end
		
		if P.flashScreen == true then
			Flash()
		end
		
		if P.soundOnly == false then
			if GetCVar_("gxWindow") == "0" then
				ConsoleExec_("gxRestart") -- /console gxRestart
			end
		end
		
		lastFocusTime = GetTime_()
		if reason then
			ATTToggleMode(false, true)
			print_(L["Alt-Tab Toggle: |cffffff00%s|r"]:format(reason))
		else
			ATTToggleMode(false)
		end

	end
end


local soundSettings = {}
function TurnSoundsOn()
	--Sound_EnableSoundWhenGameIsInBG
	soundSettings.ALL = GetCVar_("Sound_EnableAllSound")
	soundSettings.ALLVol = GetCVar_("Sound_MasterVolume")
	soundSettings.SFX = GetCVar_("Sound_EnableSFX")
	soundSettings.SFXVol = GetCVar_("Sound_SFXVolume")
	soundSettings.BG = GetCVar_("Sound_EnableSoundWhenGameIsInBG")

	SetCVar_("Sound_EnableAllSound", "1")
	SetCVar_("Sound_MasterVolume", "1")
	SetCVar_("Sound_EnableSFX", "1")
	SetCVar_("Sound_SFXVolume", "1")
	SetCVar_("Sound_EnableSoundWhenGameIsInBG", "1")--this can't be set while alt tabbed? WTF!
end

function core:TurnSoundsOff()
	if soundSettings.ALL then
		SetCVar_("Sound_EnableAllSound", soundSettings.ALL)
	end
	if soundSettings.ALLVol then
		SetCVar_("Sound_MasterVolume", soundSettings.ALLVol)
	end
	if soundSettings.SFX then
		SetCVar_("Sound_EnableSFX", soundSettings.SFX)
	end
	if soundSettings.SFXVol then
		SetCVar_("Sound_SFXVolume", soundSettings.SFXVol)
	end
	if soundSettings.BG then
		SetCVar_("Sound_EnableSoundWhenGameIsInBG", soundSettings.BG)
	end
end

function ATTToggleMode(mode, silent, reason)
	if mode == nil then mode = not ATTActive end
	
	if mode ~= true and P.alwaysActivated == true then
		Debug("ATTToggleMode", "always activated enabled...")
		return;
	end
	
	if ATTActive ~= mode then
		ATTActive = mode
		if (ATTActive == true) then
			if not silent then
				print_(L["Alt-Tab Toggle: |cff00ff00Activated|r"], "||", reason)
			end
		else
			if not silent then
				print_(L["Alt-Tab Toggle: |cffff0000Deactivated|r"], "||", reason)
			end
		end
	end
end

function InBattleground()
	local iActive, iType = IsInInstance_()
	if iActive and iType == "pvp" then
		--We're in a battlefield.
		return true
	end
	return false
end

_G.StaticPopupDialogs["ATT_ConfirmToggling"] = {
	text = L["Alt-Tab Toggle\nSome users report WoW crashing and getting error 132. If this happens, try using the 'Sound Only' option.\n See ATT's Curse/WoWAce page for Blizzard's official error 132 support link."],
	button1 = OKAY,
	timeout = 0,
	hideOnEscape = 1,
}

_G.StaticPopupDialogs["ATT_ConfirmAlwaysActivated"] = {
	text = L["Alt-Tab Toggle\nWith 'Always Activated' enabled, you may want 'Sounds Only' enabled too."],
	button1 = OKAY,
	timeout = 0,
	hideOnEscape = 1,
}

function AddSoundFile(fileName)
	if G.soundFiles[fileName] then
		return
	end
	G.soundFiles[fileName] = directory.."Sounds\\"..fileName
	LSM:Register("sound", fileName, G.soundFiles[fileName])
	print(L["%s Has been added to list, you'll need to restart WoW to hear it."]:format(fileName))
end

local flashFrame
----------------------------------------------
function Flash()							--	/run ATT.Flash()
-- Flashs the outside of the screen red,	--
-- function copied from Omen.				--
----------------------------------------------
	if not flashFrame then
		local flasher = CreateFrame("Frame", "ATTFlashFrame")
		flasher:SetToplevel(true)
		flasher:SetFrameStrata("FULLSCREEN_DIALOG")
		flasher:SetAllPoints(UIParent)
		flasher:EnableMouse(false)
		flasher:Hide()
		flasher.texture = flasher:CreateTexture(nil, "BACKGROUND")
		flasher.texture:SetTexture("Interface\\FullScreenTextures\\LowHealth")
		flasher.texture:SetAllPoints(UIParent)
		flasher.texture:SetBlendMode("ADD")
		flasher:SetScript("OnShow", function(self)
			self.elapsed = 0
			self:SetAlpha(0)
		end)
		flasher:SetScript("OnUpdate", function(self, elapsed)
			elapsed = self.elapsed + elapsed
			if elapsed < 4 then
				local alpha = elapsed % 1.3
				if alpha < 0.15 then
					self:SetAlpha(alpha / 0.15)
				elseif alpha < 0.9 then
					self:SetAlpha(1 - (alpha - 0.15) / 0.6)
				else
					self:SetAlpha(0)
				end
			else
				self:Hide()
			end
			self.elapsed = elapsed
		end)
		flashFrame = flasher
	end

	flashFrame:Show()
end

----------------
--     UI     --
----------------
CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = GAMEOPTIONS_MENU_,
			type = "group",
			order = 1,
			args={}
		},
		sounds={
			name = L["Sounds"],
			type = "group",
			order = 2,
			args={}
		},
		
	},
}

CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= ENABLE_,
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable()
		else
			core:Disable()
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.attAFKActivate = {
	name = L["AFK Activate"],
	desc = L["Activate when you go away from the keyboard (/afk), Dectivate then you're no longer afk."],
	type = "toggle",
	order	= 2,
	set = function(info,val) 
		P.attAFKActivate = val
	end,
	get = function(info) return P.attAFKActivate end
}

CoreOptionsTable.args.core.args.attAlertSound = {
	type = "toggle",	order	= 3,
	name = L["Alert Sound"],
	desc = L["Play a sound when toggling"],
	set = function(info,val) 
		P.attAlertSound = val
	end,
	get = function(info) return P.attAlertSound end
}

CoreOptionsTable.args.core.args.soundOnly = {
	name = L["Sound Only"],
	desc = L["Only play sound on toggle event, don't force WoW to foreground."],
	type = "toggle",
	order	= 4,
	set = function(info,val) 
		P.soundOnly = val
		if val == false then
			StaticPopup_Show_("ATT_ConfirmToggling")
		end
	end,
	get = function(info) return P.soundOnly end
}


CoreOptionsTable.args.core.args.attAltTabDetection = {
	name = L["Alt Tab Detection"],
	desc = L["If your FPS hasn't refreshed in 5 seconds we assume you're alt tabbed."],
	type = "toggle",
	order	= 5,
	set = function(info,val) 
		P.attAltTabDetection = val
	end,
	get = function(info) return P.attAltTabDetection end
}

CoreOptionsTable.args.core.args.ignoreWhenDND = {
	name = L["Ignore when DND"],
	desc = L["Ignore events when you're Do Not Disturb (/DND)"],
	type = "toggle",
	order	= 6,
	set = function(info,val) 
		P.ignoreWhenDND = val
	end,
	get = function(info) return P.ignoreWhenDND end
}

CoreOptionsTable.args.core.args.alwaysActivated = {
	name = L["Always Activated"],
	desc = L["Always toggle or play sound on events."],
	type = "toggle",
	order	= 7,
	set = function(info,val) 
		P.alwaysActivated = val
		if val == true and P.soundOnly == false then
			StaticPopup_Show_("ATT_ConfirmAlwaysActivated")
		end
		ATTToggleMode(val, nil, L["Always Activated"])
	end,
	get = function(info) return P.alwaysActivated end
}

CoreOptionsTable.args.core.args.enableSounds = {
	name = L["Enable Sounds"],
	desc = L["Temporarily enable sounds when toggling."],
	type = "toggle",
	order	= 8,
	set = function(info,val) 
		P.enableSounds = val
	end,
	get = function(info) return P.enableSounds end
}

CoreOptionsTable.args.core.args.flashScreen = {
	type = "toggle",	order	= 9,
	name = L["Flash Screen"],
	desc = L["Flash the outside of the screen red for 4 seconds."],
	set = function(info,val) 
		P.flashScreen = val
	end,
	get = function(info) return P.flashScreen end
}


---------
CoreOptionsTable.args.core.args.eventHeader = {
	type = "header", order	= 10,
	width	= "full",
	name	= EVENTS_LABEL_,
}


CoreOptionsTable.args.core.args.ATTBGPop = {
	type = "toggle",	order	= 11,
	name = L["BG confirm"],
	desc = L["Battleground confirm to enter"],
	set = function(info,val) 
		P.ATTBGPop = val
	end,
	get = function(info) return P.ATTBGPop end
}





CoreOptionsTable.args.core.args.ATTIdle = {
	type = "toggle",	order	= 12,
	name = L["BG Idle"],
	desc = L["Gain battleground idle debuff"],
	set = function(info, val) 
		P.ATTIdle = val
	end,
	get = function(info) return P.ATTIdle end
}



CoreOptionsTable.args.core.args.ATTBootProposal = {
	type = "toggle",	order	= 13,
	name = L["Boot Proposal"],
	desc = L["Booting someone out of a random dungeon group"],
	set = function(info,val) 
		P.ATTBootProposal = val
	end,
	get = function(info) return P.ATTBootProposal end
}



CoreOptionsTable.args.core.args.ATTDeserter = {
	type = "toggle",	order	= 14,
	name = L["Deserter Expire"],
	desc = L["Deserter debuff expires"],
	set = function(info,val) 
		P.ATTDeserter = val
	end,
	get = function(info) return P.ATTDeserter end
}



CoreOptionsTable.args.core.args.dungeonCooldown = {
	type = "toggle",	order	= 15,
	name = L["Dungeon Cooldown"],
	desc = L["Toggle when 'Dungeon Deserter' or 'Dungeon Cooldown' expires."],
	set = function(info,val) 
		P.dungeonCooldown = val
	end,
	get = function(info) return P.dungeonCooldown end
}




CoreOptionsTable.args.core.args.ATTEnemyDetected = {
	type = "toggle",	order	= 16,
	name = L["Enemy Detected"],
	desc = L["When a Enemy is detected in combatlog. (non-BG/city only)"],
	set = function(info,val) 
		P.ATTEnemyDetected = val
	end,
	get = function(info) return P.ATTEnemyDetected end
}



CoreOptionsTable.args.core.args.ATTCombat = {
	type = "toggle",	order	= 17,
	name = COMBAT_,
	desc = L["Enter combat"],
	set = function(info,val) 
		P.ATTCombat = val
	end,
	get = function(info) return P.ATTCombat end
}



CoreOptionsTable.args.core.args.ATTEnterDungeonQueue = {
	type = "toggle",	order	= 18,
	name = ENTER_DUNGEON_,
	desc = L["Toggle on enter random dungeon window"],
	set = function(info,val) 
		P.ATTEnterDungeonQueue = val
	end,
	get = function(info) return P.ATTEnterDungeonQueue end
}



CoreOptionsTable.args.core.args.ATTFlight = {
	type = "toggle",	order	= 19,
	name = L["Flight"],
	desc = L["Flight land"],
	set = function(info,val) 
		P.ATTFlight = val
	end,
	get = function(info) return P.ATTFlight end
}



CoreOptionsTable.args.core.args.fullParty = {
	type = "toggle",	order	= 20,
	name = L["Full Party"],
	desc = L["Toggle when party becomes full"],
	set = function(info,val) 
		P.fullParty = val
	end,
	get = function(info) return P.fullParty end
}



CoreOptionsTable.args.core.args.fullRaid = {
	type = "toggle",	order	= 21,
	name = L["Full Raid"],
	desc = L["Toggle when raid gets 10, 25 and 40 members."],
	set = function(info,val) 
		P.fullRaid = val
	end,
	get = function(info) return P.fullRaid end
}



CoreOptionsTable.args.core.args.ATTBGGatesOpening = {
	type = "toggle",	order	= 22,
	name = L["Gates Opening"],
	desc = L["BG Gates opening"],
	set = function(info,val) 
		P.ATTBGGatesOpening = val
	end,
	get = function(info) return P.ATTBGGatesOpening end
}



CoreOptionsTable.args.core.args.ATTInvite = {
	type = "toggle",	order	= 23,
	name = INVITE_,
	desc = L["Group invite"],
	set = function(info,val) 
		P.ATTInvite = val
	end,
	get = function(info) return P.ATTInvite end
}



CoreOptionsTable.args.core.args.ATTReadyCheck = {
	type = "toggle",	order	= 24,
	name = READY_CHECK_,
	desc = L["Group Ready Check"],
	set = function(info,val) 
		P.ATTReadyCheck = val
	end,
	get = function(info) return P.ATTReadyCheck end
}



CoreOptionsTable.args.core.args.ATTRoleCheck = {
	type = "toggle",	order	= 25,
	name = L["Role Check"],
	desc = L["Role check when group's queuing for random dungeon."],
	set = function(info,val) 
		P.ATTRoleCheck = val
	end,
	get = function(info) return P.ATTRoleCheck end
}



CoreOptionsTable.args.core.args.ATTSummon = {
	type = "toggle",	order	= 26,
	name = SUMMON_,
	desc = L["Summon requests"],
	set = function(info,val) 
		P.ATTSummon = val
	end,
	get = function(info) return P.ATTSummon end
}



CoreOptionsTable.args.core.args.ATTWhisper = {
	type = "toggle",	order	= 27,
	name = WHISPER_,
	desc = L["Receive whisper"],
	set = function(info,val) 
		P.ATTWhisper = val
	end,
	get = function(info) return P.ATTWhisper end
}



CoreOptionsTable.args.core.args.wintergraspEntry = {
	type = "toggle",	order	= 28,
	name = L["Wintergrasp Entry"],
	desc = L["Toggle when Wintergrasp entry window appears"],
	set = function(info,val) 
		P.wintergraspEntry = val
	end,
	get = function(info) return P.wintergraspEntry end
}


-----------------------------------------------------------------------
--------------------------- Sound Options -----------------------------
-----------------------------------------------------------------------

CoreOptionsTable.args.sounds.args.massSoundChange = {
     type = 'select',
	 order	= 4,
	 width = "full",
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Change All Sounds"],
     desc = L["Changing this will change all sounds"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.mainSound -- variable that is my current selection
     end,
     set = function(info, val)
		P.mainSound = val -- saves our new selection the the current one
		
		for opt in pairs(P.sounds) do 
			P.sounds[opt] = val
		end
		
     end,
}



CoreOptionsTable.args.sounds.args.eventHeader = {
	type = "header", order	= 10,
	width	= "full",
	name	= EVENTS_LABEL_,
}

CoreOptionsTable.args.sounds.args.ATTBGPopSound = {
     type = 'select',
	 order	= 11,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["BG confirm"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTBGPop -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTBGPop = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTIdleSound = {
     type = 'select',
	 order	= 12,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["BG Idle"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTIdle -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTIdle = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTBootProposalSound = {
     type = 'select',
	 order	= 13,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Boot Proposal"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTBootProposal -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTBootProposal = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTDeserterSound = {
     type = 'select',
	 order	= 14,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Deserter Expire"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTDeserter -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTDeserter = val -- saves our new selection the the current one
     end,
}
CoreOptionsTable.args.sounds.args.dungeonCooldownSound = {
     type = 'select',
	 order	= 15,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Dungeon Cooldown"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.dungeonCooldown -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.dungeonCooldown = val -- saves our new selection the the current one
     end,
}
CoreOptionsTable.args.sounds.args.ATTEnemyDetectedSound = {
     type = 'select',
	 order	= 16,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Enemy Detected"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTEnemyDetected -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTEnemyDetected = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTCombatSound = {
     type = 'select',
	 order	= 17,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = COMBAT_,
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTCombat -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTCombat = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTEnterDungeonQueueSound = {
     type = 'select',
	 order	= 18,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = ENTER_DUNGEON_,
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTEnterDungeonQueue -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTEnterDungeonQueue = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTFlightSound = {
     type = 'select',
	 order	= 19,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Flight"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTFlight -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTFlight = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.fullPartySound = {
     type = 'select',
	 order	= 20,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Full Party"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.fullParty -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.fullParty = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.fullRaidSound = {
     type = 'select',
	 order	= 21,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Full Raid"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.fullRaid -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.fullRaid = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTBGGatesOpeningSound = {
     type = 'select',
	 order	= 22,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Gates Opening"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTBGGatesOpening -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTBGGatesOpening = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTInviteSound = {
     type = 'select',
	 order	= 23,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = INVITE_,
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTInvite -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTInvite = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTReadyCheckSound = {
     type = 'select',
	 order	= 24,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = READY_CHECK_,
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTReadyCheck -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTReadyCheck = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTRoleCheckSound = {
     type = 'select',
	 order	= 25,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Role Check"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTRoleCheck -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTRoleCheck = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTSummonSound = {
     type = 'select',
	 order	= 26,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = SUMMON_,
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTSummon -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTSummon = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.ATTWhisperSound = {
     type = 'select',
	 order	= 27,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = WHISPER_,
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.ATTWhisper -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.ATTWhisper = val -- saves our new selection the the current one
     end,
}

CoreOptionsTable.args.sounds.args.wintergraspEntrySound = {
     type = 'select',
	 order	= 28,
     dialogControl = 'LSM30_Sound', --Select your widget here
     name = L["Wintergrasp Entry"],
     desc = L["Sound to play"],
     values = LSM:HashTable('sound'), -- pull in your font list from LSM
     get = function(info, val)
		return P.sounds.wintergraspEntry -- variable that is my current selection
     end,
     set = function(info, val)
		P.sounds.wintergraspEntry = val -- saves our new selection the the current one
     end,
}

---------------------------------------
CoreOptionsTable.args.sounds.args.header2 = {
	type = "header", order	= 40,
	width	= "full",
	name	= CHAT_CONFIGURATION_,
}


tempNewSound = ""
CoreOptionsTable.args.sounds.args.newSoundName = {
	type = "input",	order	= 41,
	name	= L["New Sound Name"],
	desc	= L["Move a mp3 or wav to the ATT's Sounds folder, then enter the filename here.\nex. Something.mp3"],
	set = function(info,val) 
		tempNewSound = val
	end,
	get = function(info) return tempNewSound end
}

CoreOptionsTable.args.sounds.args.addSoundName = {
	type = "execute",	order	= 42,
	name	= L["Add Sound"],
	desc	= L["Add filename to sounds list."],
	func = function(info) 
		if tempNewSound ~= "" then
			AddSoundFile(tempNewSound)
		end
		tempNewSound = ""
	end
}

tempRemoveSoundName = ""
CoreOptionsTable.args.sounds.args.removeSoundName = {
	type = 'select',	order	= 43,
	name = L["Remove Sound Name"],
	desc = L["Select a sound to remove."],
	 
	dialogControl = 'LSM30_Sound', --Select your widget here
	values = LSM:HashTable('sound'), -- pull in your font list from LSM
	
     get = function(info, val)
		return tempRemoveSoundName -- variable that is my current selection
     end,
     set = function(info, val)
		if val ~= "None" then
			tempRemoveSoundName = val -- saves our new selection the the current one
		end
     end,
}

CoreOptionsTable.args.sounds.args.removeSound = {
	type = "execute",	order	= 44,
	name	= L["Remove Sound"],
	desc	= L["Remove the selected sound from the list."],
	func = function(info) 
		if tempRemoveSoundName ~= "" then
			if G.soundFiles[tempRemoveSoundName] then
				G.soundFiles[tempRemoveSoundName] = nil
				print(L["%s will be gone the next time you load WoW."]:format(tempRemoveSoundName))
			else
				print(L["%s is from another addon, I cannot remove it."]:format(tempRemoveSoundName))
			end
		end
		tempRemoveSoundName = ""
	end
}
